/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Profile Manager.
 *
 * The Initial Developer of the Original Code is Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Goodger (03/01/00)
 *   Seth Spitzer (28/10/99)
 *   Dan Veditz <dveditz@netscape.com>
 *   Benjamin Smedberg <bsmedberg@covad.net>
 *   Jeffrey Hammel <jhammel@mozilla.com>
 *   Jonathan Griffin <jgriffin@mozilla.com>
 *   Mark Cote <mcote@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* initial implementation from
   http://mxr.mozilla.org/mozilla-central/source/toolkit/profile/content/profileSelection.js
 */

// module globals
var gStrBundle;
var gProfileService;
var gAppConfig;

/////////////////////////////////////////////////////////////////////////////
//// markup functions

/**
 * Examines the selected app in the launch menu, and if it points
 * to the default app, and no default exists, disables the start
 * button.
 */
function setStartButtonEnabledState() {
  let path;
  let menu = document.getElementById('launch-menu');
  try {
    path = menu.selectedItem.getAttribute('value');
    if (path == 'default') {
      path = ProfileManager.defaultAppPath;
    }
  }
  catch(e) {}
  document.getElementById('start').disabled = path ? false : true;
}

/**
 * Clears the launch menu and repopulates it with all the known app
 * versions, and selects the app version mapped to the selected profile,
 * if any.
 */
function populate_launch_menu() {
  let menu = document.getElementById('launch-menu');
  menu.removeAllItems();

  // setup our canvas for text measurement
  let fontfamily = window.getComputedStyle(menu, null).
                          getPropertyValue('font-family');
  let fontsize = window.getComputedStyle(menu, null).
                        getPropertyValue('font-size');
  let fontdecl = fontsize + " " + fontfamily;
  var ctx = document.getElementById('canvas').getContext('2d');
  ctx.font = fontdecl;
  var width = menu.getAttribute('width') - 20; // -20 for padding

  // add the 'default' item
  let desc = gStrBundle.getString('launchMenuItemDefaultVersion');
  menu.appendItem(desc, 'default');
  menu.selectedIndex = 0;

  // find the app for the selected profile
  let profileApp = gAppConfig.getAppForProfile(
    gProfileService.getProfile(selectedProfile()).path);

  // populate the launch menu with all known versions, and select the
  // version mapped to the current profile, if any
  for each (let anApp in gAppConfig.getAppList()) {
    let desc = gStrBundle.getFormattedString('appDesc', 
                                             [anApp.path, anApp.version]);
    // measure the width of the string we want to put in the menu; if it's
    // wider than the menu, crop it until it fits
    let itemwidth = ctx.measureText(desc).width;
    if (itemwidth > width) {
      desc = cropPathString(desc, width, ctx);
    }
    let item = menu.appendItem(desc, anApp.path);
    if (profileApp && anApp.path.toLowerCase() == profileApp.toLowerCase()) {
      menu.selectedIndex = menu.itemCount - 1;
    }
  }
  setStartButtonEnabledState();
}

/**
 * Populates the tree control with a list of profiles.
 */
function populate_tree(element, profiles) {
  // insist we have one group of treechildren -- use the first one
  var treechildren = element.getElementsByTagName('treechildren')[0];

  // save container open/closed state, so we can retain it while
  // re-populating the tree
  var containers = {};
  var treeitems = document.getElementsByTagName('treeitem');
  for (var i = 0; i < treeitems.length; i++) {
    var item = treeitems[i];
    if (item.getAttribute('properties') != 'subitem') {
      containers[item.getAttribute('value')] = item.getAttribute('open');
    }
  }

  // clear out existing children
  removeChildren(treechildren);

  // sort profiles by name; otherwise they jump around whenever the UI 
  // is redrawn
  profiles.sort(function(a, b) {
    return a.name > b.name;
  });

  // populate the tree
  for (var i in profiles) { 
    var profile = profiles[i];
    var treeitem = document.createElement('treeitem');
    treeitem.setAttribute("value", profile.name)
    var treerow = document.createElement('treerow');
    if (profile.Backups.length) {
      treeitem.setAttribute("container", "true");
      if (containers[profile.name] && containers[profile.name] == 'true') {
        treeitem.setAttribute("open", "true");
      }
    }

    // name of profile
    var cell = document.createElement('treecell');
    cell.setAttribute('label',  profile.name);
    cell.setAttribute('class', 'name')
    cell.setAttribute('properties', 'treerowimage');
    treerow.appendChild(cell);

    // application version
    cell = document.createElement('treecell');
    cell.setAttribute('editable', 'false');
    let profile_app = gAppConfig.getAppForProfile(
      gProfileService.getProfile(profile.name).path);
    if (profile_app != "Any") {
      profile_app = getAppVersion(profile_app);
    }
    else {
      profile_app = "default";
    }
    cell.setAttribute('label', profile_app);
    cell.setAttribute('class', 'version')
    treerow.appendChild(cell);
    
    // backups
    cell = document.createElement('treecell');
    cell.setAttribute('editable', 'false');
    if (profile.Backups.length) {
      cell.setAttribute('label', profile.Backups.length);
    }
    cell.setAttribute('class', 'backups')
    treerow.appendChild(cell);

    // whether profile is the default
    cell = document.createElement('treecell');
    if (profile.Default) {
      cell.setAttribute('value', 'true');
      cell.setAttribute('properties', 'isdefault');
    }
    else {
      cell.setAttribute('properties', 'isnotdefault');
    }
    cell.setAttribute('class', 'default')
    treerow.appendChild(cell);

    // whether profile is locked
    cell = document.createElement('treecell');
    cell.setAttribute('id', 'locked-' + profile.name);
    if (gProfileService.isProfileLocked(profile.name)) {
      cell.setAttribute('properties', 'plaincheck');
    }
    treerow.appendChild(cell);

    // add the row
    treeitem.appendChild(treerow);

    // add expandable rows for backups
    if (profile.Backups.length) {
      var subchildren = document.createElement('treechildren');
      for (var i in profile.Backups) {
        var subitem = document.createElement('treeitem');
        subitem.setAttribute('value', profile.name)
        subitem.setAttribute('properties', 'subitem');
        // add the backup's path as a custom attribute
        subitem.setAttribute('path', profile.Backups[i].file.path);
        var subrow = document.createElement('treerow');
        for (var j = 0; j < 2; j++) {
          var subcell = document.createElement('treecell');
          subcell.setAttribute('editable', 'false');
          subrow.appendChild(subcell);
        }
        var subcell = document.createElement('treecell');
        subcell.setAttribute('editable', 'false');
        var backupDate = profile.Backups[i].date;
        var backupString = formatDate(backupDate);
        subcell.setAttribute('label', backupString);
        subrow.appendChild(subcell);
        subitem.appendChild(subrow);
        subchildren.appendChild(subitem);
      }
      treeitem.appendChild(subchildren);
    }

    treechildren.appendChild(treeitem);
  }
  element.focus();
}

/**
 * Set the lock status of the given profile by updating the checkmark in 
 * the UI.
 */
function updateLockStatus(profileName, locked) {
  var cell = document.getElementById('locked-' + profileName);
  cell.setAttribute('properties', locked ? 'plaincheck' : '');
}

/**
 * Update button and menu states according to the selected profile.
 */
function updateButtonStates(selected_profile) {

  function grayProfileButtons(classes, state) {
    // function to enable/disable buttons that depend on a selected profile
    // - classes : string of classes to select on
    // - state : state of disabled flag
    var elements = document.getElementsByClassName(classes);
    for (var i in elements) {
      var element = elements[i];
      element.disabled = state;
    }
  }

  // preserve the selected profile
  if(selected_profile) {
    try {
      var profile = gProfileService.getProfile(selected_profile);
    } catch(e) { // selected_profile does not exist
      grayProfileButtons('requires-profile requires-backups', true);
      return;
    }

    // enable buttons requiring profiles
    grayProfileButtons('requires-profile', false);

    var state = gProfileService.isProfileLocked(selected_profile);
    updateLockStatus(selected_profile, state);

    // disable/enable buttons requiring backups
    var state = (!profile.Backups.length);
    grayProfileButtons('requires-backups', state);
  }
  else { // gray out buttons requiring the profile or backups
    grayProfileButtons('requires-profile requires-backups', true);
  }
  
}

/**
 * Center the window onscreen.
 */
function centerWindowOnScreen()
{
  window.sizeToContent();
  var xOffset = screen.availWidth/2 - window.outerWidth/2;
  var yOffset = screen.availHeight/2 - window.outerHeight/2;

  xOffset = ( xOffset > 0 ) ? xOffset : 0;
  yOffset = ( yOffset > 0 ) ? yOffset : 0;
  window.moveTo( xOffset, yOffset);
}

/**
 * Redraw all of the dynamic pieces of markup.
 */
function update_profile_markup(selected_profile) {

  // elements for the profiles
  var profilesTreeElement = document.getElementById("profile-tree");

  if (!selected_profile) {
    selected_profile = selectedProfile();
  }
  // populate the profiles elements
  populate_tree(profilesTreeElement, gProfileService.profiles);

  // preserve the selected profile
  if (selected_profile) {
    selectProfile(selected_profile);
  }
  else { // gray out the buttons
    updateButtonStates();
  }

  // add profile read errors, if any
  var element = document.getElementById("profile-errors");
  removeChildren(element);
  var caption = null;
  for (var name in gProfileService.errors) {
    if (!caption) {
      caption = document.createElement('caption');
      caption.setAttribute('label',
                           gStrBundle.getString('errorsTitle'));
      element.appendChild(caption);
    }
    var errorBox = document.createElement('hbox');
    errorBox.setAttribute('class', 'error');
    errorBox.setAttribute('id', 'errors-' + name);
    var errors = gProfileService.errors[name];
    if (errors.length == 1) {
      var description = document.createElement('description');
      description.setAttribute('class', 'label');
      description.setAttribute('value', name + ': ' + errors[0]);
      errorBox.appendChild(description);
    }
    else {
      var vbox = document.createElement('vbox');
      var description = document.createElement('description');
      description.setAttribute('value', name);
      errorBox.appendChild(description);
      for (var i in errors) {
        description = document.createElement('description');
        description.setAttribute('value', '- ' + errors[i]);
        vbox.appendChild(description);
      }
      errorBox.appendChild(vbox);
    }
    element.appendChild(errorBox);
  }

  var appDefaultElement = document.getElementById("app-default-path");
  if ('undefined' == typeof(ProfileManager.defaultApp)) { // file not found
    var app_default_string = gStrBundle.getFormattedString(
        'defaultAppNotFoundError', [ProfileManager.appName]);
  }
  else {
    var app_default_string = ProfileManager.defaultApp.path; 
  }
  appDefaultElement.value = app_default_string;
}

/**
 * Create a checkbox for any extra options in the UI; see 
 * ProfileManager._appData.extra_opts for a list of options.
 */
function extraOptionUI(extraOpt)
{
  var element = null;

  if (extraOpt instanceof ExtraOptionBool) {
    element = document.createElement("checkbox");
    element.setAttribute("id", "extraOpt" + extraOpt.name);
    extraOpt.selected = function() {
      return element.checked;
    };
  }
  return element;
}

/////////////////////////////////////////////////////////////////////////////
//// app core functions

/**
 * Checks to see if the given profile is locked.  If it is, display a warning
 * to the user that continuing the operation may be dangerous.
 * @param profileName the name of the profile to test
 * @return true if the profile isn't locked, or if the user chooses to
 *         continue, or false if the profile is locked and the user doesn't
 *         want to continue
 */
function checkForLockedProfile(profileName) {
  var retval = true;
  if (gProfileService.isProfileLocked(profileName)) {
    let msg = gStrBundle.getFormattedString('profileLockedWarning', 
                                            [profileName]);
    let title =  gStrBundle.getString('profileInUseTitle');
    retval = doAlertCheck(null, title, msg, true);
  }
  return retval;
}

/**
 * App startup: initialize variables and the UI.
 */
function startup()
{
  // parse command line options
  var commandline = window.arguments[0];
  try {
    var commandlineOptions = parseCommandline(commandline);
  } catch(e) {
    window.close();
    return;
  }

  // initialize our service singleton
  ProfileManager.init(commandlineOptions.app, 
                      commandlineOptions.profileDirectory,
                      commandlineOptions.binary);
  gProfileService = ProfileManager.profileService;
  gAppConfig = ProfileManager.appConfig;

  try {

    // get string bundles
    gStrBundle = document.getElementById("bundle_profileManager");

    // sanity check
    if (!ProfileManager.profileDirectory.exists()) {
      throw gStrBundle.getFormattedString(
        'profileDirDoesNotExistError', 
        [ProfileManager.profileDirectory.path]);
    }

    // if a command was specified on the command line, run it and be done
    if (commandlineOptions.command) {
      try {
        commandlineOptions.command.invoke(gProfileService,
                                          ProfileManager.defaultApp);
      } catch(e) {
        // TODO: set (and exit) with status code
        dump(e);
      }
      window.close();
      return;
    }

    // fix markup
    var defaultProfile = gProfileService.defaultProfile();
    if ((!defaultProfile) && (gProfileService.profiles.length)) {
      defaultProfile = gProfileService.profiles[0];
    }
    update_profile_markup(defaultProfile.name);

    // update labels involving the application name
    var appVersionElement = document.getElementById("version");
    appVersionElement.setAttribute("label",
      gStrBundle.getFormattedString('appVersionTreeColLabel',
                                    [ProfileManager.appName]));
    var startElement = document.getElementById("start");
    startElement.setAttribute("label",
      gStrBundle.getFormattedString('startAppButtonLabel',
                                    [ProfileManager.appName]));
    document.getElementById("AppVerCaption").setAttribute("label",
      gStrBundle.getFormattedString('appVerCaptionLabel', 
                                    [ProfileManager.appName]));
    document.getElementById('menu-version').setAttribute('label',
      gStrBundle.getFormattedString('menuAppVersionLabel',
                                    [ProfileManager.appName]));
    document.getElementById("manage-versions").setAttribute("label",
      gStrBundle.getFormattedString('manageAppVersButtonLabel',
                                    [ProfileManager.appName]));
    document.getElementById("whenStartDesc").appendChild(
      document.createTextNode(
        gStrBundle.getFormattedString('whenStartDesc',
                                      [ProfileManager.appName])));
    document.getElementById("launchVerDesc").appendChild(
      document.createTextNode(
        gStrBundle.getFormattedString('launchVerDesc',
                                      [ProfileManager.appName])));

    // for each 'extra option', create the necessary element and
    // attach it to the 'appOptions' element in the main window
    ProfileManager.extraOpts.forEach(function(extraOpt) {
      var extraOptionUIElem = extraOptionUI(extraOpt);
      if (!extraOptionUIElem)
        return;
      var newRow = document.createElement("row");
      newRow.setAttribute("align", "center");
      newRow.appendChild(document.createElement("separator"));
      var description = document.createElement("description");
      description.setAttribute("value", extraOpt.desc);
      newRow.appendChild(description);
      newRow.appendChild(extraOptionUIElem);
      document.getElementById("appOptions").appendChild(newRow);
    });


    // add the stopEditing handler for the 'name' column in the tree
    // see: http://mxr.mozilla.org/mozilla-central/source/toolkit/content/widgets/tree.xml#409
    document.getElementById('profile-tree').stopEditing = function() {
      if (!this._editingColumn)
        return;
      var editingColumn = this._editingColumn;
      this._editingColumn = null;
      var newName = this.inputField.value;
      try {
        var oldName = selectedProfile();
        if (checkForLockedProfile(oldName)) {
          gProfileService.renameProfile(oldName, newName);
          this.view.setCellText(this._editingRow, editingColumn, newName);
          var items = document.getElementsByTagName('treeitem');
          for (var i = 0; i < items.length; i++) {
            if (items[i].getAttribute('value') == oldName) {
              items[i].setAttribute('value', newName);
            }
          }
        }
      }
      catch(e) { showException(e); }
      this._editingRow = -1;
      this.inputField.hidden = true;
      this.inputField.value = "";
      this.removeAttribute('editing');
      update_profile_markup();
    }

    // redraw the launch menu
    populate_launch_menu();

    // center the window
    centerWindowOnScreen();

    // Only does anything if the start-app button isn't greyed out.
    startElement.focus();
  }
  catch(e) {
    alert(e);
    window.close();
    throw(e);
  }
}

/////////////////////////////////////////////////////////////////////////////
//// handlers for buttons with dialog boxes

// hash of profileActions xul dialogs
var profileActions = {
  'menu-rename': 'renameProfile.xul',
  'toolbar-copy': 'cloneProfile.xul',
  'menu-properties': 'profileInfo.xul',
  'toolbar-about': 'about.xul'
}

/**
 * Display a modal dialog with the specified chrome url, and returns
 * its output parameters.
 */
function showDialog(url, profile, extraParams) {
  // pop up the dialog box 
  var params = { in: {appName: ProfileManager.appName,
                      profile: profile,
                      gProfileService: ProfileManager.profileService,
                      ProfileManager: ProfileManager}, 
                 out: null };
  for (var key in extraParams) {
    params.in[key] = extraParams[key];
  }
  window.openDialog(url, "dlg", "modal", params);
  return params.out;
}

/**
 * Generic dialog handler
 */
function invokeProfileAction(buttonId) {

  // ensure handler exists
  var handler = profileActions[buttonId];
  if (handler == undefined) {
    throw gStrBundle.getFormattedString('actionNotDefinedError', [buttonId]);
  }

  // determine if a profile is necessary for this dialog
  var element = document.getElementById(buttonId);
  var profile = null;
  if (hasClass(element, 'requires-profile') || 
      hasClass(element, 'requires-unlocked-profile')) {

    // get selected profile
    var profileName = selectedProfile();

    // err out if nothing selected
    if (!profileName) {
      throw gStrBundle.getString('noProfileSelectedError');
    }

    var profile = gProfileService.getProfile(profileName);
    if (!profile) { 
      throw gStrBundle.getFormattedString('profileNotFoundError',
                                          [profileName]);
    }
  }

  if (hasClass(element, 'requires-unlocked-profile')) {
    if (!checkForLockedProfile(profileName))
      return;
  }

  if (handler == 'cloneProfile.xul') {
    var params = showDialog("chrome://profilemanager/content/" + handler, profile);
    if (params) {
      var busy = new busyWindow(window,
                                gStrBundle.getString("copyProfileMessage"));
      busy.onshow = function() {
        let dir = params.dir;
        if (dir.toLowerCase() == ProfileManager.profileDirectory.path.toLowerCase())
          dir = undefined;
        try {
          gProfileService.cloneProfile(profile.name, params.name, dir);
        }
        catch(e) {
          showException(e);
        }
        busy.close();
        // redraw the markup
        update_profile_markup();
      }
      busy.show();
    }
  }
  else {
    showDialog("chrome://profilemanager/content/" + handler, profile);
    // redraw the markup
    update_profile_markup();
  }
}

/**
 * Show the 'application versions' dialog.
 */
function showAppVersionDialog() {
  var newDefaultApp = gAppConfig.
    showAppVersionsDialog(window, ProfileManager.defaultAppPath);

  // set the default app as appropriate
  ProfileManager.defaultApp = 
    (newDefaultApp == null ? 
     findDefaultApp(ProfileManager.app, 
                    ProfileManager.appProperties) : 
     newDefaultApp);

  // redraw UI
  update_profile_markup();
}

/**
 * Display the 'create profile' dialog.
 */
function onCreateProfile() {
  // pop up the dialog box 
  var params = { in: {appName: ProfileManager.appName,
                      gProfileService: gProfileService,
                      ProfileManager: ProfileManager,
                      appDefault: ProfileManager.defaultApp,
                      app: ProfileManager.app,
                      apps: gAppConfig.getAppList()}, 
                 out: null };
  window.openDialog("chrome://profilemanager/content/createProfile.xul", 
                    null,
                    "modal", 
                    params);

  if (params.out) {
    // update the gAppConfig.ini file, in case any new entries were added
    gAppConfig.writeIni();
    // update the new profile's app mapping
    if (params.out.appPath) {
      gAppConfig.setAppForProfile(
                 gProfileService.getProfile(params.out.name).path,
                 params.out.appPath);
    }
    // redraw the markup
    update_profile_markup();
  }
}

////////////////////////////////////////////////////////////////////////////
//// non-dialog front-ends

/**
 * Display an exception.  For now, we just show an alert, but could
 * potentially do something more interesting here, like include
 * a stack trace, etc.
 */
function showException(msg) {
  alert(msg);
}

/**
 * Handle the 'delete' command on the context menu differently, 
 * depending on whether a profile or a backup is selected.
 */
function onMenuDelete() {
  var profileTree = document.getElementById('profile-tree');
  var currentIndex = profileTree.currentIndex;
  var selectedItem = profileTree.view
                                .getItemAtIndex(profileTree.currentIndex);
  if (selectedItem.getAttribute('properties') == 'subitem') {
    // delete a backup
    var path = selectedItem.getAttribute('path');
    if (typeof(path) == "string") {
      var profileName = selectedItem.getAttribute('value');
      if (gProfileService.deleteBackup(profileName, path)) {
        // if the delete was successful, redraw the UI
        update_profile_markup();
      }
    }
  }
  else {
    // delete a profile
    deleteProfile();
  }
}

/**
 * Restore the selected profile backup.
 */
function onMenuRestore() {
  var profileTree = document.getElementById('profile-tree');
  var currentIndex = profileTree.currentIndex;
  var selectedItem = profileTree.view
                                .getItemAtIndex(profileTree.currentIndex);
  var profileName = selectedItem.getAttribute('value');
  var backupPath = selectedItem.getAttribute('path');
  restoreProfile(profileName, backupPath);
}

/**
 * Unpack a profile from an archive and install it.
 */
function restoreFromArchive() {
  // get the user to choose an archive name
  var fp = getFilePicker();
  fp.init(window, "Open File", nsIFilePicker.modeOpen);
  fp.appendFilters(nsIFilePicker.filterAll);

  var res = fp.show();
  if (res != nsIFilePicker.returnCancel) {
    // now get user to select a profile name for the extracted profile
    var file = fp.file;
    // strip the file extension, if any, and use the result as the suggested
    // name of the profile
    var profileName = file.leafName;
    if (profileName.lastIndexOf(".") > -1)
      profileName = profileName.substring(0, profileName.lastIndexOf("."));
    var result = showDialog("chrome://profilemanager/content/cloneProfile.xul",
                            null,
                            {'newName': profileName,
                             'title': gStrBundle.getString('archiveRestoreTitle'),
                             'desc': gStrBundle.getString('archiveRestoreDescription')});
    if (!result)
      return;
    profileName = result.name;
    var profileDir = result.dir;
    if (profileDir.toLowerCase() == 
        ProfileManager.profileDirectory.path.toLowerCase())
      profileDir = null;
    var meter = new progressMeter(
                    window,
                    gStrBundle.getString('archiveRestoreProgressDialogText'));
    var canceled = false;
    meter.oncancel = function() {
      canceled = true;
    }
    meter.onshow = function() {
      gProfileService.createFromArchive(
        file,
        profileName,
        profileDir,
        function(moreData) {
          if (!moreData) {
            meter.close();
            if (!canceled) 
              update_profile_markup();
          }
          return canceled == false;
        }
      );
    }
    meter.show();
  } // if (res != nsIFilePicker.returnCancel)
}

/**
 * Backup a profile to a zip archive.
 */
function backupToArchive() {
  // exports the selected profile to a zipfile

  var profileName = selectedProfile();
  if (!profileName || !checkForLockedProfile(profileName))
    return;

  // get the user to choose an archive name
  var fp = getFilePicker();
  fp.init(window,
          gStrBundle.getString('archiveBackupFilePickerDialogTitle'),
          nsIFilePicker.modeSave);
  fp.appendFilters(nsIFilePicker.filterAll);

  var res = fp.show();
  if (res != nsIFilePicker.returnCancel) {
    var file = fp.file;
    // add a .zip extension if none was specified
    if (file.leafName.toLowerCase().indexOf(".zip") == -1) {
      let leafName = file.leafName;
      file = file.parent.clone();
      file.append(leafName + ".zip");
    }

    var meter = new progressMeter(
        window,
        gStrBundle.getString('archiveBackupProgressDialogText'));
    var canceled = false;
    meter.oncancel = function() {
      canceled = true;
    }
    meter.onshow = function() {
      gProfileService.backupToArchive(
        file, 
        selectedProfile(), 
        function(moreData) {
          if (!moreData) {
            meter.close();
            if (!canceled) 
              update_profile_markup();
          }
          return canceled == false;
        }
      );
    }
    meter.show();
  }
}

/**
 * This function is bound to the Application Version -> Other context menu;
 * it allows a user to specify an application version which isn't already
 * listed, and associate it with the selected profile.
 */
function onSelectOtherVersion() {
  // Show a file picker for selecting another binary to associated with a 
  // profile
  var thefile = chooseBinaryWithFilePicker(ProfileManager.app);
  if (thefile) {
    let apps = gAppConfig.getAppList();
    let appPresent = false;
    for each (let appVersion in apps) {
      if (appVersion.path.toLowerCase() == thefile.path.toLowerCase()) {
        appPresent = true;
      }
    }
    if (!appPresent) {
      apps.push({path: thefile.path, version: getAppVersion(thefile.path)});
    }
    gAppConfig.setAppForProfile(
        gProfileService.getProfile(selectedProfile()).path,
        thefile.path);
  }
}

/**
 * Appends a list of available backups to the 'Backup from -> backup folder'
 * context menu.
 */
function updateRestoreMenuList() {
  removeChildren(document.getElementById('restore-menu-popup'));
  var menu = document.getElementById('restore-menu-list');
  var profile = gProfileService.getProfile(selectedProfile());
  for (var i = 0; i < profile.Backups.length; i++) {
    let menuItem = menu.appendItem(profile.Backups[i].date, 
                                   profile.Backups[i].date);
    menuItem.setAttribute('oncommand', 
      'restoreProfile("' + profile.name + '", "' + 
      profile.Backups[i].file.path.replace('\\','\\\\', 'g') + '")');
  }
}

/**
 * Popuplates the Application Version context menu with a list of all known
 * application versions.
 */
function updatePopupMenuAppVersionList() {
  var profileApp = gAppConfig.getAppForProfile(selectedProfile(),
    gProfileService.getProfile(selectedProfile()).path);
  var verMenu = document.getElementById('menu-version');
  removeChildren(document.getElementById('menu-version-popup'));
  var defaultItem = verMenu.appendItem(
      gStrBundle.getString('profileVersionMenuDefaultItem'),
      "default");
  var apps = gAppConfig.getAppList();
  var defaultFound = false;
  for (let i in apps) {
    let app_path = apps[i].path;
    let item = verMenu.appendItem(apps[i].version + " [" + 
                                  apps[i].path + "]", apps[i].path);
    if (apps[i].path == profileApp) {
      item.setAttribute('checked', 'true');
      item.setAttribute('type', 'checkbox');
      defaultFound = true;
    }
    else {
      item.addEventListener('command', function() {
        gAppConfig.setAppForProfile(
            gProfileService.getProfile(selectedProfile()).path,
            app_path);
        update_profile_markup();
      }, true);
    }
  }
  if (defaultFound) {
    defaultItem.addEventListener('command', function() {
      gAppConfig.setAppForProfile(
          gProfileService.getProfile(selectedProfile()).path);
      update_profile_markup();
    }, true);
  }
  else {
    defaultItem.setAttribute('checked', 'true');
    defaultItem.setAttribute('type', 'checkbox');
  }
  let item = verMenu.appendItem(
      gStrBundle.getString('profileVersionMenuOtherItem'),
      'other');
  item.setAttribute('oncommand', 'onSelectOtherVersion();');
}

/**
 * Called when the context menu is shown, this function shows or hides
 * menu items depending on context.
 */
function onPopupShow() {
  // update the popup menu depending on context
  var profileTree = document.getElementById('profile-tree');
  var currentIndex = profileTree.currentIndex;
  var selectedItem = profileTree.view
                                .getItemAtIndex(profileTree.currentIndex);
  if (selectedItem.getAttribute('properties') == 'subitem') {
    document.getElementById('menu-rename').hidden = true;
    document.getElementById('menu-restore').hidden = false;
    document.getElementById('menu-copy').hidden = true;
    document.getElementById('menu-version').hidden = true;
    document.getElementById('menu-properties').hidden = true;
  }
  else {
    document.getElementById('menu-rename').hidden = false;
    document.getElementById('menu-restore').hidden = true;
    document.getElementById('menu-copy').hidden = false;
    document.getElementById('menu-version').hidden = false;
    document.getElementById('menu-properties').hidden = false;
    updatePopupMenuAppVersionList();
  }
}

/**
 * Handle the mousedown event on the tree element; if the event was
 * triggered over the 'default' column, mark the appropriate profile
 * as default
 */
function onTreeMouseDown(event) {
  var tree = document.getElementById("profile-tree");
  var tbo = tree.treeBoxObject;

  // get the row, col and child element at the point
  var row = { }, col = { }, child = { };
  tbo.getCellAt(event.clientX, event.clientY, row, col, child);
  if (col.value && typeof(col.value) == "object" && 
      col.value.id == "default" && row.value > -1) {
    var treeItem = tree.view.getItemAtIndex(row.value);
    if (treeItem.getAttribute('properties') == 'subitem')
      return;
    var profileName = treeItem.getAttribute('value');

    // backup the profile
    gProfileService.setDefault(profileName);

    // redraw default UI
    update_profile_markup();
  }
}

/**
 * Delete the selected profile.
 */
function deleteProfile() {
  // get the selected profile 
  var profileName = selectedProfile();

  // err out if nothing selected
  if (!profileName) {
    var errstring = "No profile selected!";
    alert(errstring);
  }

  // check if the profile is locked
  if (!checkForLockedProfile(profileName))
    return;

  var profile = gProfileService.getProfile(profileName);
  if (profile.Backups.length) {
    var msg = gStrBundle.getFormattedString(
                                    'deleteProfileAndBackupsAlertText',
                                    [profileName, profile.Backups.length]);
  }
  else {
    var msg = gStrBundle.getFormattedString(
                                    'deleteProfileAlertText',
                                    [profileName]);
  }

  var response = doAlertCheck("profilemanager.skipalertforprofiledelete",
    gStrBundle.getString('deleteProfileAlertTitle'),
    msg,
    true);
  if (!response) {
    return;
  }

  // delete the profile
  try {
    gProfileService.deleteProfile(profileName);
  }
  catch(e) {
    showException(e);
    return;
  }

  // if there is only one profile remaining, ensure it is the default
  if ((gProfileService.profiles.length == 1) &&
      (!gProfileService.profiles[0].Default)) {
    gProfileService.setDefault(gProfileService.profiles[0].name);
  }

  // redraw the UI
  update_profile_markup();

  // since we have deleted the selected profile, select the default profile
  var defaultProfile = gProfileService.defaultProfile();
  if (defaultProfile) {
    selectProfile(defaultProfile.name);
  }

}

/**
 * Restores a profile from a backup.
 * @param profileName - the name of the profile to restore
 * @param backupPath - the path of the backup to restore
 */
function restoreProfile(profileName, backupPath) {
  // find the selected backup's date
  if (!checkForLockedProfile(profileName))
    return;

  var backupDate;
  var profile = gProfileService.getProfile(profileName);
  for each (var backup in profile.Backups) {
    if (backupPath == backup.file.path) {
      backupDate = backup['date'];
    }
  }

  // do the restore
  var busy = new busyWindow(
                 window,
                 gStrBundle.getString('profileRestoreBusyWindowTitle'));
  busy.onshow = function() {
    if (backupDate) {
      try {
        gProfileService.restoreProfile(profile.name, backupDate, false);
      } catch(e) {
        busy.close();
        showException(e);
        return;
      }
    }
    busy.close();
    // since this produces no changes in the UI, we should give the user
    // confirmation that something really happened
    doAlertCheck("profilemanager.skipalertafterrestore",
                 gStrBundle.getString('profileRestoreDoneAlertTitle'),
                 gStrBundle.getFormattedString('profileRestoreDoneAlertText',
                                               [profile.name, backupDate]));
  }
  busy.show();
}

/**
 * Backup the selected profile to the backup folder.
 */
function backupProfile() {
  // get the selected profile
  var profileName = selectedProfile();

  // err out if nothing selected
  if (!profileName) {
    throw gStrBundle.getString('profileBackupNoProfileSelectedError');
  }

  if (!checkForLockedProfile(profileName))
    return;

  var busy = new busyWindow(
    window, 
    gStrBundle.getString('profileBackupBusyWindowText'));
  busy.onshow = function() {
    // backup the profile
    try {
      gProfileService.backupProfile(profileName);
    }
    catch(e) {
      showException(e);
    }
    // redraw the UI
    update_profile_markup();
    busy.close();
  }
  busy.show();
}

/////////////////////////////////////////////////////////////////////////////
// handlers for buttons that are always present

/**
 * Closes the Profile Manager.
 */
function exitDialog()
{
  window.close();
  return true;
}

/**
 * Handles the 'Start app' button.
 */
function onStartApp() {
  let menu = document.getElementById("launch-menu");
  let path = menu.selectedItem.getAttribute('value');
  if (path == 'default') {
    path = ProfileManager.defaultAppPath;
  }

  let checkbox = document.getElementById('use-temporary-profile');
  let useTemporaryProfile = checkbox.selected;

  let file = getFile(path);
  if (useTemporaryProfile) {
    var path = gProfileService.createTemporaryProfile().path;
  } else {
    // TODO: deal with possibility that no profile is selected
    // (button should be grayed out unless use-temporary-profile is selected)
    let profileName = selectedProfile();
    if (!checkForLockedProfile(profileName))
      return;
    var path = gProfileService.getProfile(profileName).rootDir.path;
  }

  var args = ['-no-remote', '-profile', path];

  ProfileManager.extraOpts.forEach(function(extra_opt) {
    var option = extra_opt.option();
    if (option)
      args.push(option);
  });

  var process = Cc["@mozilla.org/process/util;1"].
                createInstance(Ci.nsIProcess);
  try {
    process.init(file);
    process.run(false, args, args.length);
    exitDialog();
  }
  catch(e) {
    showException(e);
  }
}

////////////////////////////////////////////////////////////////////////////
//// functions for profile selection

/**
 * Selects the named profile in the main tree control.
 */
function selectProfile(profileName) { 
  var profileTable = document.getElementById('profile-tree');
  var treechildren = document.getElementById('treechildren');

  var index = 0;
  for (var i = 0; i < profileTable.view.rowCount; i++) {
    if (profileTable.view.getItemAtIndex(i).getAttribute('value') ==
        profileName && index == 0) {
      index = i;
    }
  }

  profileTable.view.selection.select(index);
}

/**
 * Handles the 'onselect' event from the tree element.
 */
function onProfileSelect() {
  // find the selected item
  var profileTree = document.getElementById('profile-tree');
  var currentIndex = profileTree.currentIndex;
  if (currentIndex < 0)
    return;
  var selectedItem = profileTree.view
                                .getItemAtIndex(profileTree.currentIndex);
  var profileName = selectedItem.getAttribute('value');
  try {
    var profile = gProfileService.getProfile(profileName);
  }
  catch(e) {
    // it's possible to select a profile that no longer exists, during e.g.
    // rename operations, before the UI is updated.  catch this case 
    // and ignore it.
    return;
  }

  // update the launch menu
  populate_launch_menu();
  // update the state of the buttons
  updateButtonStates(profileName);
}

/**
 * Returns the name of the currently selected profile.
 */
function selectedProfile() {
  var profileTree = document.getElementById('profile-tree');
  var currentIndex = profileTree.currentIndex;
  if (currentIndex < 0)
    return;
  var selectedItem = profileTree.view
                                .getItemAtIndex(profileTree.currentIndex);
  var profileName = selectedItem.getAttribute('value');
  return profileName;
}
